==> magics++_2.8.0.dfsg.orig.tar.gz: magics++-2.8.0.dfsg/src/decoders/TitleTemplate.h <==
/******************************** LICENSE ********************************

 Copyright 2007 European Centre for Medium-Range Weather Forecasts (ECMWF)

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 ******************************** LICENSE ********************************/

/*! \file TitleTemplate.h
    \brief Definition of the Template class TitleTemplate.
    
    Magics Team - ECMWF 2004
    
    Started: Mon 21-Jun-2004
    
    Changes:
    
*/

#ifndef TitleTemplate_H
#define TitleTemplate_H

#include <magics.h>
#include <TitleTemplateAttributes.h>
#include <TitleField.h>
#include <Data.h>
#include <Factory.h>
#include <Title.h>
#include <VectorOfPointers.h>

#include <stack>

namespace magics {


class SimpleStringFormat
{
public:
	SimpleStringFormat(const string& value, const string& format) : format_(format), value_(value) {}
protected:
	string format_;
	string value_;
	
	void operator()(ostream& out) const
	{
	  string dup = format_;
	  string::size_type pos = dup.find ("%s",0);

	  if (pos != string::npos)
	  {
	    dup.replace(pos,2,value_);
	  }
	  out << dup;
	}

	friend ostream& operator<<(ostream& out,const SimpleStringFormat& format)
	{
	      format(out);
	      return out;
	}
};

template <class D>
class TitleFieldHandler 
{
public :
    TitleFieldHandler() {}
    virtual ~TitleFieldHandler() {};
    virtual void operator()(TitleField& field, ostream& out, const D&)
    {
        out << field; 
    }
};


class TitleTemplate : public VectorOfPointers<vector<TitleTemplate*> >, 
    public TitleTemplateAttributes, 
    public std::stack<TitleTemplate*> {

public:
	TitleTemplate(map<string, string> criteria) : criteria_(criteria) {}
	TitleTemplate();
	virtual ~TitleTemplate();

	void add(TitleField* field) { template_.push_back(field); }

	template <class D>
	static void title(ostream& out,  const D& data)
	{
		if ( !singleton_) new TitleTemplate();
		(*singleton_)(out, data);
	}

	static void release() 
	{
		if (singleton_) {
		     delete singleton_;
		     singleton_ = 0;
		}
	}

	template <class D>
	void operator()(ostream& out, const D& data) const
	{
		if ( !verify(data) ) return;
		for (const_iterator child = begin(); child != end(); ++child)
		{
			if ((*child)->verify(data))
			{
				(*(*child))(out, data);
				return;
			}
		}

		for (vector<TitleField*>::const_iterator entry = template_.begin(); entry != template_.end(); ++entry)
		{
#ifdef MAGICS_EXCEPTION
			try
			{
				auto_ptr<TitleFieldHandler<D> > object(SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name()));
				(*object)(*(*entry), out, data);
			}
			catch (NoFactoryException& e)
			{
				// The data do not know how to verify the criter ....
				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
				(*(*entry))(out);
			}
#else
			TitleFieldHandler<D>* object = SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name());
			if (object)
			{
				(*object)(*(*entry), out, data);
			}
			else
			{
				// The data do not know how to verify the criter ....
				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
				(*(*entry))(out);
			}
#endif
		}
	}

/*!
 \todo code needed if MAGICS_EXCEPTION are disabled!
*/
	template <class D>
	bool verify(const D& data) const
	{
		for (map<string, string>::const_iterator criter = criteria_.begin(); criter != criteria_.end(); ++criter)
		{
//#ifdef MAGICS_EXCEPTION
			try{
				Log::debug() << "Try  to create the MatchCriteria for " << criter->first << "\n";
				auto_ptr<MatchCriteria<D> >  object(SimpleObjectMaker<MatchCriteria<D> >::create(criter->first));
				Log::debug() << "Found the MatchCriteria for " << criter->first << "\n";
				if (!(*object).verify(data, criter->first, criter->second)) return false;
			}
			catch (NoFactoryException& e) 
			{ // The data do not know how to verify the criter ....
				Log::warning() << "Can Not Create the MatchCriteria for " << criter->first << "\n";
				return false; 
			}
//#else
			
//#endif
		}
		return true;
	}

	map<string, string>& criteria() { return criteria_; }

protected:
     //! Method to print string about this class on to a stream of type ostream (virtual).
	virtual void print(ostream&) const; 
	map<string, string> criteria_;
	VectorOfPointers<vector<TitleField*> > template_;
	static TitleTemplate* singleton_;
	void decode();

private:
	//! Copy constructor - No copy allowed
	TitleTemplate(const TitleTemplate&);
	//! Overloaded << operator to copy - No copy allowed
	TitleTemplate& operator=(const TitleTemplate&);

// -- Friends
    //! Overloaded << operator to call print().
	friend ostream& operator<<(ostream& s,const TitleTemplate& p)
		{ p.print(s); return s; }
};

} // namespace magics
#endif

==> magics++_2.10.0.dfsg.orig.tar.gz: magics++-2.10.0.dfsg/src/decoders/TitleTemplate.h <==
/******************************** LICENSE ********************************

 Copyright 2007 European Centre for Medium-Range Weather Forecasts (ECMWF)

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 ******************************** LICENSE ********************************/

/*! \file TitleTemplate.h
    \brief Definition of the Template class TitleTemplate.
    
    Magics Team - ECMWF 2004
    
    Started: Mon 21-Jun-2004
    
    Changes:
    
*/

#ifndef TitleTemplate_H
#define TitleTemplate_H

#include <magics.h>
#include <TitleTemplateAttributes.h>
#include <TitleField.h>
#include <Data.h>
#include <Factory.h>
#include <Title.h>
#include <VectorOfPointers.h>

#include <stack>

namespace magics {


class SimpleStringFormat
{
public:
	SimpleStringFormat(const string& value, const string& format) : format_(format), value_(value) {}
protected:
	string format_;
	string value_;
	
	void operator()(ostream& out) const
	{
	  string dup = format_;
	  string::size_type pos = dup.find ("%s",0);

	  if (pos != string::npos)
	  {
	    dup.replace(pos,2,value_);
	  }
	  out << dup;
	}

	friend ostream& operator<<(ostream& out,const SimpleStringFormat& format)
	{
	      format(out);
	      return out;
	}
};

template <class D>
class TitleFieldHandler 
{
public :
    TitleFieldHandler() {}
    virtual ~TitleFieldHandler() {};
    virtual void operator()(TitleField& field, ostream& out, const D&)
    {
        out << field; 
    }
};


class TitleTemplate : public VectorOfPointers<vector<TitleTemplate*> >, 
	public TitleTemplateAttributes, 
	public std::stack<TitleTemplate*>
{

public:
	TitleTemplate(map<string, string> criteria) : criteria_(criteria) {}
	TitleTemplate();
	virtual ~TitleTemplate();

	void add(TitleField* field) { template_.push_back(field); }

	template <class D>
	static void title(ostream& out,  const D& data)
	{
		if ( !singleton_) new TitleTemplate();
		(*singleton_)(out, data);
	}

	static void release() 
	{
		if (singleton_) {
		     delete singleton_;
		     singleton_ = 0;
		}
	}

	template <class D>
	void operator()(ostream& out, const D& data) const
	{
		if ( !verify(data) ) return;
		for (const_iterator child = begin(); child != end(); ++child)
		{
			if ((*child)->verify(data))
			{
				(*(*child))(out, data);
				return;
			}
		}

		for (vector<TitleField*>::const_iterator entry = template_.begin(); entry != template_.end(); ++entry)
		{
#ifdef MAGICS_EXCEPTION
			try
			{
				auto_ptr<TitleFieldHandler<D> > object(SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name()));
				(*object)(*(*entry), out, data);
			}
			catch (NoFactoryException& e)
			{
				// The data do not know how to verify the criter ....
				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
				(*(*entry))(out);
			}
#else
			TitleFieldHandler<D>* object = SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name());
			if (object)
			{
				(*object)(*(*entry), out, data);
			}
			else
			{
				// The data do not know how to verify the criter ....
				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
				(*(*entry))(out);
			}
#endif
		}
	}

/*!
 \todo code needed if MAGICS_EXCEPTION are disabled!
*/
	template <class D>
	bool verify(const D& data) const
	{
		for (map<string, string>::const_iterator criter = criteria_.begin(); criter != criteria_.end(); ++criter)
		{
//#ifdef MAGICS_EXCEPTION
			try{
				Log::debug() << "Try  to create the MatchCriteria for " << criter->first << "\n";
				auto_ptr<MatchCriteria<D> >  object(SimpleObjectMaker<MatchCriteria<D> >::create(criter->first));
				Log::debug() << "Found the MatchCriteria for " << criter->first << "\n";
				if (!(*object).verify(data, criter->first, criter->second)) return false;
			}
			catch (NoFactoryException& e) 
			{ // The data do not know how to verify the criter ....
				Log::warning() << "Can Not Create the MatchCriteria for " << criter->first << "\n";
				return false; 
			}
//#else
			
//#endif
		}
		return true;
	}

	map<string, string>& criteria() { return criteria_; }

protected:
     //! Method to print string about this class on to a stream of type ostream (virtual).
	virtual void print(ostream&) const; 
	map<string, string> criteria_;
	VectorOfPointers<vector<TitleField*> > template_;
	static TitleTemplate* singleton_;
	void decode();

private:
	//! Copy constructor - No copy allowed
	TitleTemplate(const TitleTemplate&);
	//! Overloaded << operator to copy - No copy allowed
	TitleTemplate& operator=(const TitleTemplate&);

// -- Friends
    //! Overloaded << operator to call print().
	friend ostream& operator<<(ostream& s,const TitleTemplate& p)
		{ p.print(s); return s; }
};

} // namespace magics
#endif
